/*
 * FloorLight.c
 *
 * Created: 22.11.2016 06:29:38
 *  
 */

#include "FloorLight.h"
 

#include <avr/io.h>
#include <avr/pgmspace.h>

#define F_CPU 16000000L
#include <util/delay.h>

#define STK500 0

#if STK500
// inverted PWM on OC1A for STK500
#define INVERT_PWM (1 << COM1A0)
#else
// non-inverted PWM on OC1A
#define INVERT_PWM 0
#endif // STK500

#define FADE_MAX 64

floorLight_state_t floorLightState;
int16_t floortmp;

void initFloorLight()
{
	// 10 Bit Fast PWM
	TCCR1A = 0x83 | INVERT_PWM;
	// prescaler 64 -> ~122 Hz PWM frequency
	TCCR1B = (1 << WGM12) | 3;
	
	DDRB |= 1 << PINB1;
	floorLightState = STATE_OFF;
	floortmp = 0;
}

 
const uint16_t pwmtable_10[64] PROGMEM =
{
    0, 1, 1, 2, 2, 2, 2, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6, 7, 8, 9, 10,
    11, 12, 13, 15, 17, 19, 21, 23, 26, 29, 32, 36, 40, 44, 49, 55,
    61, 68, 76, 85, 94, 105, 117, 131, 146, 162, 181, 202, 225, 250,
    279, 311, 346, 386, 430, 479, 534, 595, 663, 739, 824, 918, 1023
};
 


/*

  Diese Tabellen sind nicht nach der Theorie (s. oben) berechnet, sondern wie folgt:
  a = Anzahl an Schritte (4, 8, 16, 32, 64, 256)
  b = Auflsung des PWM's (256, 1024, 65536)
  y = Errechneter Wert an einer stelle x
      y = 0 wenn x = 0
      y = pow(2, log2(b-1) * (x+1) / a) wenn x > 0

  Gerne wird auch diese alternative Formel genutzt:
      y = pow(2, log2(b) * (x+1) / a) - 1

  Obige Funktionen wurden in den LibreOffice Calc Sheet eingearbeitet.
*/
 
// long, variable delays
 
void fadeFloorLightOn()
{
	
	if(floorLightState == STATE_FADE_ON )
	{
		OCR1A = pgm_read_word (& pwmtable_10[floortmp++]);
		if (floortmp >= 64)
		{
			floorLightState = STATE_ON;
			floortmp = 63;
		}
	}
	
}

void fadeFoorLightOFF()
{
	if(floorLightState == STATE_FADE_OFF)
	{
		OCR1A = pgm_read_word (& pwmtable_10[floortmp--]);
		if (floortmp < 0)
		{
			floorLightState = STATE_OFF;
			floortmp = 0;
		}
	}
}

void floorLightTask()
{
	 fadeFloorLightOn();
	 fadeFoorLightOFF();
	 
}
 
void setFloorLigt(floorLight_state_t state)
{
	if(state == STATE_ON)
	{
		floortmp = 63;
		 state = STATE_FADE_ON;
	}
	
	if(state == STATE_OFF)
	{
		floortmp = 1;
		 state = STATE_FADE_OFF;
	}
	
	floorLightState = state;
}

floorLight_state_t getFloorLightState()
{
	return floorLightState;
}

 
