/*
 * RS485.c
 *
 * Created: 22.11.2016 14:09:05
 *  
 */ 
#include <avr/io.h>
#include <avr/interrupt.h>

#include "RS485.h"
#include "globalDef.h"
#include "crc16.h"

st_Dev_Config *Dev_Config;

st_protokoll_Data stRxData;
st_protokoll_Data stTxData;

unsigned char *NetTxPtr;
unsigned char NetTxCount;
unsigned char NetTxCount_temp;

void NetSetTxEnable(bool enable);

void NetMPCMode(bool mode)
{
	if(Dev_Config->MasterMode) return;
	
	if(mode)NET_UCSRA |= (1 << MPCM0);
	else NET_UCSRA &= ~(1 << MPCM0); 	
}

void NetInit(st_Dev_Config *config) {
	char c;
	Dev_Config = config;
	UBRR0H = (unsigned char)(UART_BAUD_CALC(Dev_Config->speed, F_CPU) >> 8);
	UBRR0L = (unsigned char)UART_BAUD_CALC(Dev_Config->speed, F_CPU);

	NET_UCSRA = (1 << U2X0);
	NET_UCSRB = (1 << UCSZ02) | (1 << RXCIE0) | (1 << RXEN0) | (1 << TXEN0) | (1 << TXCIE0);
	NET_UCSRC = (1 << UCSZ01) | (1 << UCSZ00);
	DDRD |= (1 << PIND2);
	NetSetTxEnable(false);
	NetMPCMode(true);
	c = NET_UDR; // Rx Buffer leeren und verwerfen
}


void NetSetTxEnable(bool enable) {
	if (enable) {
		PORTD |= (1 << PIND2);
	}
	else {
		PORTD &= ~(1 << PIND2);
	}
}

void Net_SendFrame(void)
{
	uCRC16_t CRC;
	if(INCLUDE_CRC(stTxData.InfoByte))
	{
		stTxData.DataLength += (INCLUDE_CRC(stTxData.InfoByte));			// + 2 Byte CRC
		CRC = CalcCRCBuffer(&stTxData.RxAddress,stTxData.DataLength +2);	// + 2 Byte Header (normal 4 aber da CRC schon +2 ist aber nicht berechnet wird reichen 2 aus)
		stTxData.Data[stTxData.DataLength-(CRC_HI+1)] = CRC.cCRC16[CRC_HI]; // CRC Hi einfgen
		stTxData.Data[stTxData.DataLength-(CRC_LO+1)] = CRC.cCRC16[CRC_LO]; // CRC lo einfgen
	}
	NetTxCount = 4 + stTxData.DataLength;
	NetTxCount_temp = NetTxCount;
	NetTxPtr = &stTxData.RxAddress;
	NetSetTxEnable(true);
	/*if(Dev_Config->MasterMode)*/NET_UCSRB |= (1<< TXB80); // nur beim Master
	//
	//else NET_UCSRB &= ~(1<< TXB80);
	NET_UCSRB |= (1<<UDRIE0); // Tx Empty einschalten
	
}

void Net_SendChar(unsigned char ptr_RXAddress,unsigned char *val,unsigned int len, unsigned char infoByte)
{
	int loop;
	stTxData.RxAddress = ptr_RXAddress;
	stTxData.TxAddress = Dev_Config->MyAdr;
	stTxData.InfoByte = infoByte;
	stTxData.DataLength = len;
	for(loop = 0; loop < len; loop++){
		stTxData.Data[loop] = *val++;
	}
	Net_SendFrame();
}

void Net_SendAck(unsigned char ptr_RXAddress)
{
	stTxData.RxAddress = ptr_RXAddress;
	stTxData.TxAddress = Dev_Config->MyAdr;
	stTxData.InfoByte = ACK_BIT | CRC_BIT;
	stTxData.DataLength = 0;
	Net_SendFrame();
}
void Net_RunOnAck (fptrAck func)
{
	Dev_Config->fptrCallBackOnAck = func;
}

void HandleNewFrame(void)
{
	if( !ACK_FRAME( stRxData.InfoByte ) )Dev_Config->fptrCallBackOnOk(&stRxData);
	else {
		if(Dev_Config->fptrCallBackOnAck != NULL) Dev_Config->fptrCallBackOnAck () ;
		Dev_Config->fptrCallBackOnAck = NULL;
	}
}


ISR(USART_UDRE_vect) // Tx Buffer ist leer und braucht nachschub
{
	
	if (NetTxCount)
	{
		if(NetTxCount != NetTxCount_temp)NET_UCSRB &= ~(1<< TXB80);
		NET_UDR = *NetTxPtr;
		NetTxPtr++;
		NetTxCount--;
	}
	else
	NET_UCSRB &= ~(1<<UDRIE0); // Tx Empty abschalten
}


ISR(USART_TX_vect)  // Leztes Byte wurde gesendet
{
	NetSetTxEnable(false);
	NOP;
	NOP;
	
}

ISR(USART_RX_vect) {
	unsigned char ucRxByte;
	static unsigned char RxBody = 0 ;
	static net_rx_state_t net_rx_state = STATE_ADR;
	
	if (NET_UCSRA & ((1 << FE0) + (1 << DOR0))) // Fehler
	{
		ucRxByte = NET_UDR; // Rx Buffer leeren
		NetMPCMode(true); // MPCM Mode einschalten
		return;
	}

	if ((NET_UCSRB & (1 << RXB80)) ) // OK mit 9. bit
	{

		ucRxByte = NET_UDR;
		if (ucRxByte == Dev_Config->MyAdr) {
			NetMPCMode(false); // MPCM Mode ausschalten
			stRxData.RxAddress = ucRxByte;
			net_rx_state = STATE_INFO;
			return;
		}
		else // Packet ist nicht fr mich gedacht.
		{
			NetMPCMode(true);
			return;
		}

	}
	// ab hier geht es weiter mit 8 bit
	ucRxByte = NET_UDR;
	switch(net_rx_state)
	{
		case STATE_INFO:
		stRxData.InfoByte = ucRxByte;
		net_rx_state =STATE_SENDER;
		break;
		case STATE_SENDER:
		stRxData.TxAddress = ucRxByte;
		net_rx_state =STATE_LENGHT;
		break;
		case STATE_LENGHT:
		stRxData.DataLength = ucRxByte;
		net_rx_state =STATE_DATA;
		RxBody = 0;
		break;
		case STATE_DATA:
		if (RxBody < stRxData.DataLength ) // DATA
		{
			stRxData.Data[RxBody] = ucRxByte;
			RxBody ++;
			if (RxBody == stRxData.DataLength)
			{
				if (!INCLUDE_CRC(stRxData.InfoByte)) // Keine CRC ntig
				{
					NetMPCMode(true);
					HandleNewFrame();
					return;
				}
				if (!CheckCRC(&stRxData.RxAddress,stRxData.DataLength+2)) // fehler CRC
				{
					NetMPCMode(true);
					return;
				}
				else // alles OK
				{
					NetMPCMode(true);
					HandleNewFrame();
					return;
				}
			}
			return;
		}
		break;
		
		default:
		break;
	}

	if (RxBody==MAX_DATA_LENGTH)
	{
		NetMPCMode(true);
		return;
	}
	return;
}