/*
 * RS485.h
 *
 * Created: 22.11.2016 14:09:21
 *  
 */ 


#ifndef RS485_H_
#define RS485_H_

#include <stdbool.h>
#include <stddef.h>

#define UART_BAUD_CALC(UART_BAUD_RATE,F_OSC) \
( ( F_OSC ) / ( ( UART_BAUD_RATE ) * 8UL ) - 1 )


#define NET_UCSRA             UCSR0A
#define NET_UCSRB             UCSR0B
#define NET_UCSRC             UCSR0C
#define NET_UDR               UDR0
#define CRC_BIT 			  0x10
#define ACK_BIT				  0x20
#define NEED_ACK			  0x01

#define INCLUDE_CRC(x)	    	(((x) & (CRC_BIT)) !=0) ? (2):(0)
#define ACK_FRAME(x)			(((x) & (ACK_BIT)) !=0)
#define SEND_ACK(x)			(((x) & (NEED_ACK)) !=0)

#define MAX_DATA_LENGTH 10

typedef struct {
	unsigned char RxAddress;
	unsigned char InfoByte;
	unsigned char TxAddress;
	unsigned char DataLength;
	unsigned char Data[MAX_DATA_LENGTH];
} st_protokoll_Data;

typedef union{
	float 			fVal;
	unsigned int 	iVal;
	unsigned char  	cVal[4];
}multiVal_t;

typedef void ( *fptrNetTask)(st_protokoll_Data*);
typedef void ( *fptrAck)(void);

typedef struct{
	unsigned char MyAdr;
	bool MasterMode;
	unsigned int speed;
	fptrNetTask	fptrCallBackOnOk;
	fptrAck		fptrCallBackOnAck;
}st_Dev_Config;

typedef enum {
	STATE_ADR = 0,
	STATE_INFO,
	STATE_SENDER,
	STATE_LENGHT,
	STATE_DATA,
	STATE_END = 0xff
}net_rx_state_t;



void NetInit(st_Dev_Config *config);
void Net_SendChar(unsigned char ptr_RXAddress,unsigned char *val,unsigned int len, unsigned char infoByte);
void Net_SendAck(unsigned char ptr_RXAddress);
void Net_RunOnAck (fptrAck func);




#endif /* RS485_H_ */